// C Header File
// Created 21/01/2003; 00:22:44

#ifndef __complib__
#define __complib__

/*===============================================================================================
== Error codes return by the function with the pararmeter error
=================================================================================================
== if error<0, it fails : no creation of handle
           >0, the creation is done, but there is a error
           =0 : no errors
===============================================================================================*/
typedef enum {
  COMP_NO_ERR                =  0,  //no problem
  COMP_ERR_MEM               = -1,  //memory error
  COMP_ERR_CANT_ADD          = -2,  //impossible to add a variable
  COMP_ERR_NO_SRC_VAR        = -3,  //the source variable doesn't exist
  COMP_ERR_NOT_A_COMP        = -4,  //the datas are not compressed
  COMP_ERR_WRONG_TYPE        = -5,  //a type is needed
  COMP_ERR_CANT_COMP         = -6,  //the compressed datas are bigger than the original ones
  COMP_ERR_DATA              = -7,  //decompressed data > original data
  COMP_ERR_NOT_SUPPORTED_ZIP = -8,  //the compressed zip have more than one file compressed
  COMP_ERR_CANT_REARCH       =  1  //the file cannot be rearchive
} CompErr;

/*===============================================================================================
== Some definitions of function type
=================================================================================================
== FctComplib_draw_message : this type of function will be used in parameter of the function
DrawCompError. You have to give a function that can draw a text : here, a message about the
compression
== FctComplib_comp : this type will be used in the function WorkOnFolder. You have to give the
function that will be used on every file of the folder : this will be a function of compression
or decompression of a HSym structure in the same variable.
=================================================================================================
== Important note : if the function that you give calls some ROM fuctions, don't compile it with
the OPTIMIZE_ROM_CALLS macro, else it will crash !
===============================================================================================*/
typedef void (*FctComplib_draw_message) (const unsigned char * message) ;
typedef HSym (*FctComplib_comp)(HSym sym_var_src, const unsigned char * nom_var_dest, CompErr * error);


/************************************************************************************************
*************************************************************************************************
** For the compression, you can choose the different format : komp or ziplib                   **
** For the decompression, you can choose the different format : komp, ziplib or comp           **
comp is generic : it ckecks the format and decompress the data with the right decompressor     **
*************************************************************************************************
************************************************************************************************/


/*===============================================================================================
== Evaluate the size of the compressed datas
=================================================================================================
== data : pointer of the datas to compress
== length : length of the datas to compress
== var : indicates if you want to make a var, or a temporary handle
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern unsigned short eval_zip_length(unsigned char * data, unsigned short length, BOOL var);
extern unsigned short eval_komp_length(unsigned char * data, unsigned short length, BOOL var);

/*===============================================================================================
== Compression of datas at a specifyied place
=================================================================================================
== data : pointer of the datas to compress
== length : length of the datas to compress
== data_dest : pointer of the destination datas
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern CompErr zip_data(unsigned char * data, unsigned short length, unsigned char * data_dest, unsigned char type);
extern CompErr komp_data(unsigned char * data, unsigned short length, unsigned char * data_dest, unsigned char type);

/*===============================================================================================
== Compression of datas in a temporary handle
=================================================================================================
== data : pointer of the datas to compress
== length : length of the datas to compress
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern HANDLE zip_data_temp(unsigned char * data, unsigned short length, unsigned char type, CompErr * error);
extern HANDLE komp_data_temp(unsigned char * data, unsigned short length, unsigned char type, CompErr * error);

/*===============================================================================================
== Compression of datas in a variable
=================================================================================================
== data : pointer of the datas to compress
== length : length of the datas to compress
== name_var_dest : name of the variable where the compressed datas will be stored
== str_tag : a string which contain the tag to add atthe end of the file. For exemple :
'\0','E','X','T','\O',OTH_TAG
== tag_length : length of the tag (the tag doesn't finish by '\0' ! impossible to use strlen !)
== error : pointer to return the error code
=================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/
extern HSym zip_data_hsym(unsigned char * data, unsigned short length, const unsigned char * nom_var, CompErr * error);
extern HSym komp_data_hsym(unsigned char * data, unsigned short length, const unsigned char * nom_var, CompErr * error);

/*===============================================================================================
== Compression of a variable at a specifyied place
=================================================================================================
== hsym :
== data_dest : pointer of the destination datas
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern CompErr zip_hsym(HSym hsym_var_src, unsigned char * data_dest);
extern CompErr komp_hsym(HSym hsym_var_src, unsigned char * data_dest);

/*===============================================================================================
== Compression of a variable in a temporary handle
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern HANDLE zip_hsym_temp(HSym hsym_var_src, CompErr * error);
extern HANDLE komp_hsym_temp(HSym hsym_var_src, CompErr * error);

/*===============================================================================================
== Compression of a variable in an other (or the same) variable
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== name_var_dest : name of the variable where the compressed datas will be stored
         if it's NULL, compress in the same variable
== error : pointer to return the error code
==================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/
extern HSym zip_hsym_var(HSym sym_var_src, const unsigned char * nom_var_dest, CompErr * error);
extern HSym komp_hsym_var(HSym sym_var_src, const unsigned char * nom_var_dest, CompErr * error);

/*===============================================================================================
== Decompression of data in a temprorary handle
=================================================================================================
== data : pointer of the data to decompress
== length : length of the data to decompress
== error : pointer to return the error code
=================================================================================================
== return the handle of the decompressed datas
===============================================================================================*/
extern CompErr dzip_data(unsigned char * data, unsigned short length, unsigned char * dest);
extern CompErr dkomp_data(unsigned char * data, unsigned short length, unsigned char * dest);

/*===============================================================================================
== Decompression of data in a temprorary handle
=================================================================================================
== data : pointer of the data to decompress
== length : length of the data to decompress
== error : pointer to return the error code
=================================================================================================
== return the handle of the decompressed datas
===============================================================================================*/
extern HANDLE dzip_data_temp(unsigned char * data, unsigned short length, CompErr * error);
extern HANDLE dkomp_data_temp(unsigned char * data, unsigned short length, CompErr * error);

/*===============================================================================================
== Decompression of datas in a variables
=================================================================================================
== data : pointer of the data to decompress
== length : length of the data to decompress
== name_var_dest : name of the variable where the compressed datas will be stored
== str_tag : a string which contain the tag to add atthe end of the file. For exemple :
'\0','E','X','T','\O',OTH_TAG
== tag_length : length of the tag (the tag doesn't finish by '\0' ! impossible to use strlen !)
== error : pointer to return the error code
=================================================================================================
== return the hsym structure created, else HS_NULL
===============================================================================================*/
extern HSym dzip_data_hsym(unsigned char * data, unsigned short length, const unsigned char * nom_var_dest, const unsigned char * str_tag, unsigned char tag_length, CompErr * error);
extern HSym dkomp_data_hsym(unsigned char * data, unsigned short length, const unsigned char * nom_var_dest, const unsigned char * str_tag, unsigned char tag_length, CompErr * error);

/*===============================================================================================
== Decompression of a variable in a temporary handle
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern CompErr dzip_hsym(HSym hsym_var_src, unsigned char * dest);
extern CompErr dkomp_hsym(HSym hsym_var_src, unsigned char * dest);

/*===============================================================================================
== Decompression of a variable in a temporary handle
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern HANDLE dzip_hsym_temp(HSym hsym_var_src, CompErr * error);
extern HANDLE dkomp_hsym_temp(HSym hsym_var_src, CompErr * error);

/*===============================================================================================
== Decompression of a variable in a temporary handle
=================================================================================================
== hsym_var_src : HSym structure of the var to compress
== error : pointer to return the error code
=================================================================================================
== return a handle with the compressed datas : the H_NULL handle if there is a error.
===============================================================================================*/
extern HSym dzip_hsym_var(HSym hsym_var_src, const unsigned char * nom_var_dest, CompErr * error);
extern HSym dkomp_hsym_var(HSym hsym_var_src, const unsigned char * nom_var_dest, CompErr * error);

/************************************************************************************************
*************************************************************************************************
** The following functions are some utils on compressed datas                                  **
*************************************************************************************************
************************************************************************************************/

/*===============================================================================================
== Structure that describe the compressed var : if the var is not compressed, the fields are fill
with the variable information.
===============================================================================================*/

typedef struct {
  unsigned char var;    //type of the initial var (Ti-Os codes)
  unsigned char comp;   //type of compression
  unsigned short size;  //size of the initial var
} InfosComp;

/*===============================================================================================
== type of compression stored in the comp field
===============================================================================================*/
#define COMP_TYPE_NORMAL  0
#define COMP_TYPE_ZIPLIB  1
#define COMP_TYPE_KOMP    2
#define COMP_TYPE_LZFO    3

/*===============================================================================================
== Give the infomation of datas
=================================================================================================
== data : pointer of the datas
=================================================================================================
== return the InfosComp structure
===============================================================================================*/
extern InfosComp GetInfosComp(unsigned char * data);

/*===============================================================================================
== Many macros to traduce the information contained in the InfosComp structure
=================================================================================================
== is_compressed return TRUE if the datas are compressed
=================================================================================================
== is_compressed_ziplib return TRUE if the datas are compressed with ziplib
=================================================================================================
== is_compressed_komp return TRUE if the datas are compressed with komp
=================================================================================================
== get_compress_type return the code of the compression
=================================================================================================
== get_var_type return the type of the initial variable
=================================================================================================
== get_size_var return the initial size of the var
===============================================================================================*/
#define is_compressed(infoscomp)            (infoscomp.comp!=COMP_TYPE_NORMAL)
#define is_compressed_ziplib(infoscomp)     (infoscomp.comp==COMP_TYPE_ZIPLIB)
#define is_compressed_komp(infoscomp)       (infoscomp.comp==COMP_TYPE_KOMP)
#define get_compress_type(infoscomp)        (infoscomp.comp)
#define get_var_type(infoscomp)             (infoscomp.var)
#define get_size_var(infoscomp)             (infoscomp.size)

/*===============================================================================================
== Other macros : the HSym and Sym* version of GetInfosComp
===============================================================================================*/
#define GetInfosComp_sym(SymPtr)     GetInfosComp(HeapDeref(SymPtr->handle))
#define GetInfosComp_hsym(hsym)      GetInfosComp(HeapDeref(DerefSym(hsym)->handle))

/*===============================================================================================
== Write the errors
=================================================================================================
== error : the error code
== draw_progress : the function which draw the message
== draw : the priority : see DRAW defines
===============================================================================================*/
#define DRAW_ALL      0  //draw every message : error, warning and succes
#define DRAW_WARNING  1  //draw if there is a error or a waring
#define DRAW_ERRORS   2  //draw only errors
extern void DrawCompErrorGB(char error, FctComplib_draw_message draw_message, char draw);
extern void DrawCompErrorFR(char error, FctComplib_draw_message draw_message, char draw);

/*===============================================================================================
== Generic functions wich (de)compress every files in a folder
=================================================================================================
== name_fold : name of the folder
== fct_comp : function of compression or decompression to use on the files
== draw_message : the function which draw the message. It can be NULL, and the sensibility of
drawing message is defined by the parameter draw
== draw_progress_global : function that will will show the evolution of the action on the entire
folder. In fact, is count the files. It can be NULL.
== draw_progress_local : function that will show tghe progress of the compression of each file. It
can be NULL
== draw : the priority of drawing the messages: see DRAW defines
===============================================================================================*/
extern CompErr * WorkOnFolderGB(const unsigned char * name_folder, FctComplib_comp fct_comp, FctComplib_draw_message draw_message, char draw);
extern CompErr * WorkOnFolderFR(const unsigned char * name_folder, FctComplib_comp fct_comp, FctComplib_draw_message draw_message, char draw);

#define ZipOnFolderGB(name_fold,draw_message,draw)         WorkOnFolderGB(name_fold,zip_hsym_var,draw_message,draw)
#define KompOnFolderGB(name_fold,draw_message,draw)        WorkOnFolderGB(name_fold,komp_hsym_var,draw_message,draw)

#define dZipOnFolderGB(name_fold,draw_message,draw)        WorkOnFolderGB(name_fold,dzip_hsym_var,draw_message,draw)
#define dKompOnFolderGB(name_fold,draw_message,draw)       WorkOnFolderGB(name_fold,dkomp_hsym_var,draw_message,draw)

#define ZipOnFolderFR(name_fold,draw_message,draw)         WorkOnFolderFR(name_fold,zip_hsym_var,draw_message,draw)
#define KompOnFolderFR(name_fold,draw_message,draw)        WorkOnFolderFR(name_fold,komp_hsym_var,draw_message,draw)

#define dZipOnFolderFR(name_fold,draw_message,draw)        WorkOnFolderFR(name_fold,dzip_hsym_var,draw_message,draw)
#define dKompOnFolderFR(name_fold,draw_message,draw)       WorkOnFolderFR(name_fold,dkomp_hsym_var,draw_message,draw)

extern unsigned char conv_type_filelib_TIOS[15];

/*===============================================================================================
== Give the filelib type corresponding to the Ti-Os type
=================================================================================================
== type : TI-OS type
=================================================================================================
== rrturn the filelib type
===============================================================================================*/
extern unsigned char conv_type_TIOS_filelib(unsigned char type);

/*===============================================================================================
== Create a Ti-OS variable
=================================================================================================
== hsym_var_src : hsym pointer to the source variable
== name_var_dest : string of the name of the variable to create : if NULL, it replaces the source
variable
== h : handle to associate to the new variable
== error : pointer to return the error code
=================================================================================================
== return the HSym structure of the new var
===============================================================================================*/
extern HSym mk_var(HSym hsym_var_src, const char * name_var_dest, HANDLE h, CompErr * error);

/*===============================================================================================
== Give the sym pointer of the folder of the variable pointed by a sym pointer (Thanks to Kevin
Kofler)
=================================================================================================
== SymPtr : sym pointer of the variable
=================================================================================================
== return the sym pointer of the folder
===============================================================================================*/
extern SYM_ENTRY * find_Sym_fold(SYM_ENTRY * SymPtr);

#endif
